package javax.security.auth.kerberos;

import sun.security.util.DerValue;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.security.krb5.Realm;
import sun.security.krb5.KrbException;
import sun.security.krb5.PrincipalName;
import java.io.Serializable;
import java.security.Principal;

public final class KerberosPrincipal implements Principal, Serializable
{
    private static final long serialVersionUID = -7374788026156829911L;
    public static final int KRB_NT_UNKNOWN = 0;
    public static final int KRB_NT_PRINCIPAL = 1;
    public static final int KRB_NT_SRV_INST = 2;
    public static final int KRB_NT_SRV_HST = 3;
    public static final int KRB_NT_SRV_XHST = 4;
    public static final int KRB_NT_UID = 5;
    private transient String fullName;
    private transient String realm;
    private transient int nameType;
    
    public KerberosPrincipal(String s) {
        this(s, 1);
    }
    
    public KerberosPrincipal(String name, int n) {
        PrincipalName principalName;
        try {
            principalName = new PrincipalName(name, n);
        }
        catch (KrbException ex) {
            throw new IllegalArgumentException(ex.getMessage());
        }
        if (principalName.isRealmDeduced() && !Realm.AUTODEDUCEREALM) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                try {
                    securityManager.checkPermission(new ServicePermission("@" + principalName.getRealmAsString(), "-"));
                }
                catch (SecurityException ex2) {
                    throw new SecurityException("Cannot read realm info");
                }
            }
        }
        this.nameType = n;
        this.fullName = principalName.toString();
        this.realm = principalName.getRealmString();
    }
    
    public String getRealm() {
        return this.realm;
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o instanceof KerberosPrincipal && this.getName().equals(((KerberosPrincipal)o).getName()));
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        try {
            PrincipalName principalName = new PrincipalName(this.fullName, this.nameType);
            objectOutputStream.writeObject(principalName.asn1Encode());
            objectOutputStream.writeObject(principalName.getRealm().asn1Encode());
        }
        catch (Exception cause) {
            throw new IOException(cause);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        byte[] encoding = (byte[])(byte[])objectInputStream.readObject();
        byte[] encoding2 = (byte[])(byte[])objectInputStream.readObject();
        try {
            Realm realm = new Realm(new DerValue(encoding2));
            PrincipalName principalName = new PrincipalName(new DerValue(encoding), realm);
            this.realm = realm.toString();
            this.fullName = principalName.toString();
            this.nameType = principalName.getNameType();
        }
        catch (Exception cause) {
            throw new IOException(cause);
        }
    }
    
    @Override
    public String getName() {
        return this.fullName;
    }
    
    public int getNameType() {
        return this.nameType;
    }
    
    @Override
    public String toString() {
        return this.getName();
    }
}
